package com.scenegraphshell;

import javax.swing.*;
import java.awt.*;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.lang.reflect.Method;

/**
 * The main program. Shows scenegraph editor and preview. Add a menu for loading predefined scenegraphs.
 */
public class ScenegraphShell {
    public static enum SourceType{
        BeanShell,
        Java
    }
    private ScenegraphPreviewComponent sceneGraphPreviewComponent = new ScenegraphPreviewComponent();

    private JMenuBar menuBar = new JMenuBar();


    private JMenu beanShellMenu = new JMenu("BeanShell");
    private JMenu javaMenu = new JMenu("Java");
    private static String EDITOR_PREFIX = "// The following variables are available in the interpreter:\n" +
                    "// import com.sun.scenario.scenegraph.*; // import scenegraph statements\n" +
                    "// import com.sun.scenario.animation.*;; // import animation statements\n" +
                    "// root - SGGroup, the root node the user nodes should be attached to\n" +
                    "// timeline - Timeline, animations should be added to this timeline to be executed\n";
    private ScenegraphEditor scenegraphEditor = new ScenegraphEditor(EDITOR_PREFIX);
    private final static String[][] beanShellCodeExamples = new String[][]{
            {"New",null},
            {"Piechart demo","/src/piechart.beanshell"},
            {"Text demo","/src/textdemo.beanshell"},
    };
    private final static String[][] javaCodeExamples = new String[][]{
            {"New","/src/new.java.txt"},
            {"Piechart demo","/src/piechart.java.txt"},
            {"Text demo","/src/textdemo.java.txt"},
    };

    private SourceType currentSourceType = null;
    private ScenegraphCompiler scenegraphCompiler;

    public ScenegraphShell() {
        final JFrame frame = new JFrame("Scenegraph Shell 0.2");

        scenegraphCompiler = new ScenegraphCompiler(scenegraphEditor, sceneGraphPreviewComponent);

        frame.setJMenuBar(menuBar);
        menuBar.add(javaMenu);
        menuBar.add(beanShellMenu);
       
        for (int i=0;i< beanShellCodeExamples.length;i++){
            JMenuItem newMenu = new JMenuItem(beanShellCodeExamples[i][0]);
            beanShellMenu.add(newMenu);
            final String resourceUrl = beanShellCodeExamples[i][1];
            ActionListener actionListener = new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    final String code = EDITOR_PREFIX+(resourceUrl==null?"":loadCode(ScenegraphShell.this.getClass().getResourceAsStream(resourceUrl), SourceType.BeanShell));
                    scenegraphEditor.setCode(code);
                }
            };
            newMenu.addActionListener(actionListener);
            if (i==1){
                actionListener.actionPerformed(null); // invoke an example
            }
        }

        final boolean isJWS = isJWS();
        for (int i=0;i<javaCodeExamples.length;i++){
            JMenuItem newMenu = new JMenuItem(javaCodeExamples[i][0]);
            javaMenu.add(newMenu);
            final String resourceUrl = javaCodeExamples[i][1];

            newMenu.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if (isJWS){
                        JOptionPane.showMessageDialog(frame, "<html>Java compilation is currently not working in Java WebStart<br>Download the jar to see the java compiler in action</html>");
                    }
                    else{
                        final String code = (resourceUrl==null?"":loadCode(ScenegraphShell.this.getClass().getResourceAsStream(resourceUrl), SourceType.Java));
                        scenegraphEditor.setCode(code);
                    }
                }
            });
        }

        Container container = frame.getContentPane();
        container.setLayout(new BorderLayout());
        JSplitPane splitPane = new JSplitPane(JSplitPane.VERTICAL_SPLIT);
        container.add(splitPane, BorderLayout.CENTER);

        splitPane.setTopComponent(sceneGraphPreviewComponent);
        splitPane.setBottomComponent(scenegraphEditor);
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setSize(800,800);
        frame.setVisible(true);
        splitPane.setDividerLocation(.5);

    }

    static public boolean isJWS(){
        try {
            Class classServiceManager = Class.forName("javax.jnlp.ServiceManager");
            Method m = classServiceManager.getMethod("lookup",String.class);
            return m.invoke(null, "javax.jnlp.FileOpenService")!=null;
        } catch (Exception e) {
            return false;
        }
    }

    private String loadCode(InputStream resourceAsStream, SourceType sourceType) {
        StringBuilder sb = new StringBuilder();
        try{
            InputStreamReader isr = new InputStreamReader(resourceAsStream);
            BufferedReader br = new BufferedReader(isr);

            String s = br.readLine();
            while (s!=null){
                sb.append(s).append('\n');
                s = br.readLine();
            }
            resourceAsStream.close();

            if (currentSourceType!=sourceType){
                currentSourceType = sourceType;
                switch (currentSourceType){
                    case BeanShell:
                        scenegraphEditor.setTitle("BeanShell script editor");
                        scenegraphCompiler.setCompilerInterface(new BeanShellInterpreter());
                        break;
                    case Java:
                        scenegraphEditor.setTitle("Java editor");
                        scenegraphCompiler.setCompilerInterface(new EclipseMemoryCompiler());
                        break;
                }

            }
        }
        catch (Exception e){
            e.printStackTrace();
        }
        return sb.toString();
    }

    public static void main(String[] args) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try{
                    UIManager.setLookAndFeel("com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel");
                }
                catch (Exception e){
                    // ignore
                    System.out.println("Nimbus not supported");
                    try {
                        UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
                    } catch (Exception e1) {
                        // ignore
                    }
                }
                new ScenegraphShell();
            }
        });
    }
}
